@@ -79,6 +79,7 @@ public:
     QHash<QByteArray, int> roles;
     ActionStateParser* actionStateParser;
     QHash<UnityMenuAction*, GtkSimpleActionObserver*> registeredActions;
+    bool destructorGuard;
 
     static void nameAppeared(GDBusConnection *connection, const gchar *name, const gchar *owner, gpointer user_data);
     static void nameVanished(GDBusConnection *connection, const gchar *name, gpointer user_data);
@@ -113,6 +114,7 @@ UnityMenuModelPrivate::UnityMenuModelPrivate(UnityMenuModel *model)
     this->connection = NULL;
     this->nameWatchId = 0;
     this->actionStateParser = new ActionStateParser(model);
+    this->destructorGuard = false;
 
     this->muxer = gtk_action_muxer_new ();
 
@@ -126,6 +128,7 @@ UnityMenuModelPrivate::UnityMenuModelPrivate(const UnityMenuModelPrivate& other,
     this->connection = NULL;
     this->nameWatchId = 0;
     this->actionStateParser = new ActionStateParser(model);
+    this->destructorGuard = false;
 
     this->muxer = GTK_ACTION_MUXER( g_object_ref(other.muxer));
 
@@ -134,6 +137,7 @@ UnityMenuModelPrivate::UnityMenuModelPrivate(const UnityMenuModelPrivate& other,
 
 UnityMenuModelPrivate::~UnityMenuModelPrivate()
 {
+    this->destructorGuard = true;
     this->clearItems(false);
 
     g_sequence_free(this->items);
@@ -144,6 +148,7 @@ UnityMenuModelPrivate::~UnityMenuModelPrivate()
     QHash<UnityMenuAction*, GtkSimpleActionObserver*>::const_iterator it = this->registeredActions.constBegin();
     for (; it != this->registeredActions.constEnd(); ++it) {
         g_object_unref(it.value());
+        it.key()->setModel(NULL);
     }
     this->registeredActions.clear();
 
@@ -165,6 +170,8 @@ void UnityMenuModelPrivate::clearName()
 
     this->updateActions();
     this->updateMenuModel();
+
+    Q_EMIT model->nameOwnerChanged (this->nameOwner);
 }
 
 void UnityMenuModelPrivate::updateActions()
@@ -226,6 +233,8 @@ void UnityMenuModelPrivate::nameAppeared(GDBusConnection *connection, const gcha
 
     priv->updateActions();
     priv->updateMenuModel();
+
+    Q_EMIT priv->model->nameOwnerChanged (priv->nameOwner);
 }
 
 void UnityMenuModelPrivate::nameVanished(GDBusConnection *connection, const gchar *name, gpointer user_data)
@@ -290,8 +299,16 @@ QByteArray UnityMenuModel::busName() const
     return priv->busName;
 }
 
+QByteArray UnityMenuModel::nameOwner() const
+{
+    return priv->nameOwner;
+}
+
 void UnityMenuModel::setBusName(const QByteArray &name)
 {
+    if (name == priv->busName)
+        return;
+
     priv->clearName();
 
     if (priv->nameWatchId)
@@ -301,6 +318,7 @@ void UnityMenuModel::setBusName(const QByteArray &name)
                                           UnityMenuModelPrivate::nameAppeared, UnityMenuModelPrivate::nameVanished,
                                           priv, NULL);
     priv->busName = name;
+    Q_EMIT busNameChanged (priv->busName);
 }
 
 QVariantMap UnityMenuModel::actions() const
@@ -367,7 +385,7 @@ static QString iconUri(GIcon *icon)
         GFile *file;
 
         file = g_file_icon_get_file (G_FILE_ICON (icon));
-        if (g_file_is_native (file)) {
+        if (file) {
             gchar *fileuri;
 
             fileuri = g_file_get_uri (file);
@@ -784,6 +802,9 @@ bool UnityMenuModel::event(QEvent* e)
 
 void UnityMenuModel::registerAction(UnityMenuAction* action)
 {
+    if (priv->destructorGuard)
+        return;
+
     if (!priv->registeredActions.contains(action)) {
         GtkSimpleActionObserver* observer_item;
         observer_item = gtk_simple_action_observer_new(GTK_ACTION_OBSERVABLE (priv->muxer),
@@ -805,6 +826,9 @@ void UnityMenuModel::registerAction(UnityMenuAction* action)
 
 void UnityMenuModel::unregisterAction(UnityMenuAction* action)
 {
+    if (priv->destructorGuard)
+        return;
+
     if (priv->registeredActions.contains(action)) {
         GtkSimpleActionObserver* observer_item;
         observer_item = priv->registeredActions[action];
